@@ -553,12 +553,25 @@ def getArrayDimPathFromType(rtype):
     else:
         return DimPath()
 
-def getNestedArraySubType(rtype):
+def getNestedArraySubType(rtype,depth):
+    if depth == 0:
+        return rtype
     if(rtype.__class__ is rtypes.TypeArray):
-        return getNestedArraySubType(self.getSubType())
+        depth -= len(rtype.dims)
+        if depth < 0:
+            return rtype
+        else:
+            return getNestedArraySubType(rtype.getSubType(),depth)
     else:
         return rtype
-   
+
+def dimsToArrays(dims, subtype):
+    dims = dims[::-1]
+    subtype = rtypes.TypeArray(dims=dims[:1],subtypes=(subtype,))
+
+    for pos in xrange(1,len(dims)):
+        subtype = rtypes.TypeArray(dims=dims[pos:(pos+1)],subtypes=(subtype,))
+    return subtype
 
 class DimPathRoot(Dim):
     pass
@@ -599,10 +612,10 @@ def identifyUniqueDimPath(source,dim_selector):#{{{
 def identifyDimPath(sourcepaths, dim_selector):#{{{
    
     if(isinstance(dim_selector, int)):
-        udpath = uniqueDimPath(sourcepaths)
+        udpath = uniqueDimPath(sourcepaths,only_unique=False)
         dim_selector = udpath[dim_selector]
     elif(isinstance(dim_selector, long)):
-        udpath = uniqueDimPath(sourcepaths)
+        udpath = uniqueDimPath(sourcepaths,only_unique=False)
         if(dim_selector < 0):
             dim_selector += len(udpath)
         if not (dim_selector >= 0 and dim_selector < len(udpath)):
