@@ -7,31 +7,24 @@ import repops_funcs
 _delay_import_(globals(),"utils","util","context")
 _delay_import_(globals(),"ops")
 _delay_import_(globals(),"representor")
-_delay_import_(globals(),"wrappers","wrapper_py")
+_delay_import_(globals(),"wrappers","python")
 _delay_import_(globals(),"itypes","rtypes","dimensions","dimpaths")
 _delay_import_(globals(),"repops_dim")
 _delay_import_(globals(),"repops_slice")
 
 class Broadcast(repops.MultiOpRep):
-    def _process(self,sources, mode="dim"):
-        state = reduce(operator.__and__,[source._state for source in sources])
-        if not state & RS_SLICES_KNOWN:
-            return
-
+    def _sprocess(self,sources, mode="dim"):
         nslices = []
         for bcslices in util.zip_broadcast(*[source._slices for source in sources]):
             nslices.extend(ops.broadcast(bcslices,mode)[0])
-        return self._initialize(tuple(nslices),state)
+        return self._initialize(tuple(nslices))
 
 
 class Nest(repops.MultiOpRep):
     def __init__(self, lsource, rsource, dim=None):
         repops.MultiOpRep.__init__(self,(lsource,rsource),dim=dim)
-    def _process(self, sources, dim=None):
+    def _sprocess(self, sources, dim=None):
         assert len(sources) == 2, "Nest expects two representor objects"
-        state = reduce(operator.__and__,[source._state for source in sources])
-        if not state & RS_SLICES_KNOWN:
-            return
         lsource,rsource = sources
         
         joinpath = dimpaths.identifyUniqueDimPathSource(lsource, dim)
@@ -58,17 +51,14 @@ class Nest(repops.MultiOpRep):
                 slice = ops.InsertDimOp(slice,dimpos,idims[dimpos])
             slice = ops.BroadcastOp(slice,references,plan,joinpath + odims)
             nslices.append(slice)
-        return self._initialize(tuple(nslices), state)
+        return self._initialize(tuple(nslices))
 
 class Combine(repops.MultiOpRep):
     def __init__(self, *sources):
         repops.MultiOpRep.__init__(self, sources)
-    def _process(self,sources):
-        state = reduce(operator.__and__,[source._state for source in sources])
-        if not state & RS_SLICES_KNOWN:
-            return
+    def _sprocess(self,sources):
         nslices = self._apply(*[source._slices for source in sources])
-        return self._initialize(nslices,state)
+        return self._initialize(nslices)
 
     @classmethod
     def _apply(cls, *xslicelists):
@@ -86,7 +76,7 @@ class Group(repops.MultiOpRep):
 
     def _process(self,sources, flat):
         source, gsource = sources 
-        if not source._state & RS_SLICES_KNOWN or not gsource._state & RS_TYPES_KNOWN:
+        if not source._slicesKnown() or not gsource._typesKnown():
             return
 
         gslices = [ops.ensure_frozen(slice) for slice in gsource._slices]
@@ -108,7 +98,7 @@ class Group(repops.MultiOpRep):
                 keyslices = gsource.Get(key)
             nkey = tuple([gsource._slices.index(keyslice) for keyslice in keyslices._slices])
              
-            if(isinstance(values,tuple)):
+            if(isinstance(values,(list,tuple))):
                 vslices = source.Get(*values)
             else:
                 vslices = source.Get(values)
@@ -123,7 +113,10 @@ class Group(repops.MultiOpRep):
 
         #group slices
         for pos, slice in enumerate(gsource._slices):
-            p = oslices.index(slice)
+            try:
+                p = oslices.index(slice)
+            except ValueError:
+                continue
             key = (pos,)
             if(key in xflat):
                 xflat[key].add(p)
@@ -142,7 +135,7 @@ class Group(repops.MultiOpRep):
         #filter flattened slices 
         gidx = range(len(gsource._slices))
         gdims = gslice.dims[-len(gsource._slices):]
-        firstelem = wrapper_py.Rep(0)._slices[0]
+        firstelem = python.Rep(0)._slices[0]
         for key, pos in xflat.iteritems():
             tslice = gslice
             for elem in gidx:
@@ -158,20 +151,18 @@ class Group(repops.MultiOpRep):
             for p,fslice in zip(pos,fslices):
                 nslices[p] = fslice
             
-        return self._initialize(tuple(nslices),source._state)
+        return self._initialize(tuple(nslices))
 
 
 class Filter(repops.MultiOpRep):
     def __init__(self,source,constraint,dim=None):
         if(not isinstance(constraint,representor.Representor)):
-            constraint = repops.PlusPrefix(wrapper_py.Rep(constraint,name="filter"))
+            constraint = repops.PlusPrefix(python.Rep(constraint,name="filter"))
         repops.MultiOpRep.__init__(self,(source,constraint),dim=dim)
 
     def _process(self,sources,dim):
         source,constraint = sources
-        if not source._state & RS_SLICES_KNOWN:
-            return
-        if not constraint._state & RS_TYPES_KNOWN:
+        if not source._slicesKnown() or not constraint._typesKnown():
             return
         assert len(constraint._slices) == 1, "Filter constraint should have 1 slice"
         cslice = constraint._slices[0]
@@ -186,7 +177,7 @@ class Filter(repops.MultiOpRep):
 
         nslices = self._apply(source._slices,cslice,seldimpath,mode)
 
-        return self._initialize(tuple(nslices),source._state)
+        return self._initialize(tuple(nslices))
 
     @classmethod
     def _apply(cls,fslices,cslice,seldimpath, bcmode):
@@ -226,13 +217,10 @@ class Sort(repops.MultiOpRep):
             repops.MultiOpRep.__init__(self,(source,),descend=descend)
         else:
             if(not isinstance(constraint,representor.Representor)):
-                constraint = repops.PlusPrefix(wrapper_py.Rep(constraint,name="filter"))
+                constraint = repops.PlusPrefix(python.Rep(constraint,name="filter"))
             repops.MultiOpRep.__init__(self,(source,constraint),descend=descend)
 
-    def _process(self, sources, descend):
-        if not any([s._state & RS_SLICES_KNOWN for s in sources]):
-            return
-
+    def _sprocess(self, sources, descend):
         if(len(sources) == 1): #no explicit constraint, use data itself
             source = sources[0]
             if len(source._slices) > 1:
@@ -249,7 +237,7 @@ class Sort(repops.MultiOpRep):
         constraint = repops_funcs.Argsort(constraint, descend=descend)
         cslice = ops.PackArrayOp(constraint._slices[0])
         nslices = Filter._apply(source._slices, cslice, cslice.type.dims[:1],"dim")
-        return self._initialize(tuple(nslices),source._state & constraint._state)
+        return self._initialize(tuple(nslices))
 
 
 class Unique(repops.MultiOpRep):
@@ -258,13 +246,10 @@ class Unique(repops.MultiOpRep):
             repops.MultiOpRep.__init__(self,(source,),descend=descend)
         else:
             if(not isinstance(constraint,representor.Representor)):
-                constraint = repops.PlusPrefix(wrapper_py.Rep(constraint,name="filter"))
+                constraint = repops.PlusPrefix(python.Rep(constraint,name="filter"))
             repops.MultiOpRep.__init__(self,(source,constraint),descend=descend)
 
-    def _process(self, sources, descend):
-        if not any([s._state & RS_SLICES_KNOWN for s in sources]):
-            return
-
+    def _sprocess(self, sources, descend):
         if(len(sources) == 1): #no explicit constraint, use data itself
             source = sources[0]
             if len(source._slices) > 1:
@@ -282,7 +267,7 @@ class Unique(repops.MultiOpRep):
         uconstraint = repops_funcs.Argunique(constraint)
         cslice = uconstraint._slices[0]
         nslices = Filter._apply(source._slices, cslice, dpath[-1:],"dim")
-        return self._initialize(tuple(nslices),source._state & constraint._state)
+        return self._initialize(tuple(nslices))
 
 
 class Join(repops.MultiOpRep):
@@ -294,8 +279,7 @@ class Join(repops.MultiOpRep):
 
     def _process(self, sources):
         lsource, rsource, constraint = sources
-        if not lsource._state & RS_SLICES_KNOWN or not rsource._state & RS_SLICES_KNOWN \
-           or not constraint._state & RS_TYPES_KNOWN:
+        if not lsource._slicesKnown() or not rsource._slicesKnown() or not constraint._typesKnown():
             return
         
         ldims = dimpaths.createDimSet([s.dims for s in lsource._slices])
@@ -335,26 +319,30 @@ class Join(repops.MultiOpRep):
             nslices = Combine._apply(rslices,lslices)
         else:
             nslices = Combine._apply(lslices,rslices)
-        return self._initialize(tuple(nslices),lsource._state & rsource._state)
+        return self._initialize(tuple(nslices))
+
 
 
 class Match(repops.MultiOpRep):
-    def __init__(self, lsource, rsource, lslice=None, rslice=None, jointype="inner"):
+    def __init__(self, lsource, rsource, lslice=None, rslice=None, jointype="inner", merge_same=False):
         assert jointype in set(["inner","left","right","full"]), "Jointype should be inner, left, right or full"
         assert (not isinstance(lslice,representor.Representor)), "Representor objects not allowed as lslice. Use context, string or int to indicate slice in lsource"
         assert (not isinstance(rslice,representor.Representor)), "Representor objects not allowed as rslice. Use context, string or int to indicate slice in rsource"
+        
+        if not rslice is None:
+            rslice = rsource.Get(rslice)
+        elif not lslice is None:
+            rslice = rsource.Get(lslice)
 
         if not lslice is None:
             lslice = lsource.Get(lslice)
-        if not rslice is None:
-            rslice = rsource.Get(rslice)
-        repops.MultiOpRep.__init__(self,(lsource,rsource,lslice,rslice),jointype=jointype)
+            
+        repops.MultiOpRep.__init__(self,(lsource,rsource,lslice,rslice),jointype=jointype, merge_same=merge_same)
 
-    def _process(self, sources, jointype):
+    def _process(self, sources, jointype, merge_same):
         lsource, rsource, lslice,rslice = sources
-        if not lsource._state & RS_SLICES_KNOWN or not rsource._state & RS_SLICES_KNOWN:
+        if not lsource._slicesKnown() or not rsource._slicesKnown():
             return
-         
         if(lslice is None):
             if(rslice is None):
                 common_names = set(lsource.Names) & set(rsource.Names)
@@ -369,14 +357,16 @@ class Match(repops.MultiOpRep):
             rslice = getattr(rsource,lslice.Names[0])
         assert len(rslice._slices) == 1, "rslice parameter in match should have only one slice"
         assert len(lslice._slices) == 1, "lslice parameter in match should have only one slice"
-
         self._sources = (lsource, rsource, lslice, rslice)
-        if not lslice._state & RS_TYPES_KNOWN or not rslice._state & RS_TYPES_KNOWN:
+        if not lslice._typesKnown() or not rslice._typesKnown():
             return
-        
         lslice = lslice._slices[0]
         rslice = rslice._slices[0]
-
+        nslices = self._apply(lsource, rsource, lslice, rslice, jointype, merge_same)
+        return self._initialize(tuple(nslices))
+   
+    @classmethod
+    def _apply(cls,lsource, rsource, lslice,rslice, jointype="inner", merge_same=False):
         lindex,rindex = ops.EquiJoinIndexOp(lslice,rslice, jointype=jointype).results
 
         lslices = list(lsource._slices)
@@ -384,23 +374,47 @@ class Match(repops.MultiOpRep):
         if(lslice.name == rslice.name and jointype=="inner" and lslice in lslices and rslice in rslices):
             rslices.pop(rslices.index(rslice))
 
-        lslices = Filter._apply(lslices, lindex, lslice.dims, "dim")
-        rslices = Filter._apply(rslices, rindex, rslice.dims, "dim")
+        lslices = list(Filter._apply(lslices, lindex, lslice.dims, "dim"))
+        rslices = list(Filter._apply(rslices, rindex, rslice.dims, "dim"))
+        if merge_same:
+            lnames = [lslice.name for lslice in lslices]
+            for rslice in list(rslices):
+                if not rslice.name in lnames:
+                    continue
+                lpos = lnames.index(rslice.name)
+                lslice = lslices[lpos]
+                lslices[lpos] = repops_funcs.Merge._apply([lslice,rslice], "dim")
+                del rslices[rslices.index(rslice)]
+        
+        return Combine._apply(lslices,rslices)
 
+class Replace(repops.MultiOpRep):
+    def __init__(self, source, slice, translator, fromslice=0, toslice=1):
+        slice = source.Get(slice)
+        fromslice = translator.Get(fromslice)
+        toslice = translator.Get(toslice)
+        translator = Combine(fromslice, toslice)
 
-        nslices = Combine._apply(lslices,rslices)
-        return self._initialize(tuple(nslices),lsource._state & rsource._state)
+        repops.MultiOpRep.__init__(self,(source, slice, translator))
+   
+    def _sprocess(self, sources):
+        source, slice, translator = sources
+        assert len(slice._slices) == 1, "Only one slice can be replaced"
+        assert len(translator._slices) == 2, "Translator should have only two selected slices"
 
+        slice = slice._slices[0]
+        slicepos = source._slices.index(slice)
+
+        nslices = list(Match._apply(source, translator, slice, translator._slices[0],"inner"))
+        nslices[slicepos] = ops.ChangeNameOp(nslices[-1],slice.name)
+        nslices = nslices[:-2]
+        return self._initialize(tuple(nslices))
 
 class Stack(repops.MultiOpRep):
     def __init__(self, *sources, **kwargs):
         repops.MultiOpRep.__init__(self,sources, **kwargs)
 
-    def _process(self, sources, dim=None):
-        state = reduce(operator.__and__,[source._state for source in sources])
-        if not state & RS_SLICES_KNOWN:
-            return
-
+    def _sprocess(self, sources, dim=None):
         slicelens = set([len(source._slices) for source in sources])
         assert len(slicelens) == 1, "Sources of stack should have same number of slices"
         nslice = slicelens.pop()
@@ -436,7 +450,7 @@ class Stack(repops.MultiOpRep):
                 res = ops.ChangeDimOp(res, len(res.dims) - dimdepth, ndim)
             nslices.append(res)
 
-        return self._initialize(tuple(nslices),RS_CHECK)
+        return self._initialize(tuple(nslices))
 
 
 
@@ -447,11 +461,7 @@ class Intersect(repops.MultiOpRep):
     def __init__(self, *sources, **kwargs):
         repops.MultiOpRep.__init__(self,sources, **kwargs)
 
-    def _process(self, sources, dim=None):
-        state = reduce(operator.__and__,[source._state for source in sources])
-        if not state & RS_SLICES_KNOWN:
-            return
-
+    def _sprocess(self, sources, dim=None):
         slicelens = set([len(source._slices) for source in sources])
         assert len(slicelens) == 1, "Sources of stack should have same number of slices"
         nslice = slicelens.pop()
@@ -496,7 +506,7 @@ class Intersect(repops.MultiOpRep):
                 pslice = ops.UnpackArrayOp(pslice, mpd)
             nslices.append(pslice)
 
-        return self._initialize(tuple(nslices),RS_CHECK)
+        return self._initialize(tuple(nslices))
 
 class Union(Intersect):
     opercls = repops_funcs.Or
