@@ -235,7 +235,7 @@ class PyExec(VisitorFactory(prefixes=("visit",), flags=NF_ELSE),
         return slice.modify(data=ndata,rtype=node.type,dims=node.dims)
 
     def visitPackArrayOp(self, node, slice):
-        ndata=slice.data.pack(node.type, len(node.type.dims))
+        ndata=slice.data.pack(node.type, len(node.pack_dims))
         return slice.modify(data=ndata,rtype=node.type,dims=node.dims)
 
     def visitInsertDimOp(self,node,slice):
@@ -251,7 +251,7 @@ class PyExec(VisitorFactory(prefixes=("visit",), flags=NF_ELSE),
         return slice.modify(data=ndata,dims=node.dims,rtype=node.type)
 
     def visitPackListOp(self, node, slice):
-        ndata=slice.data.pack(node.type, len(node.type.dims))
+        ndata=slice.data.pack(node.type, len(node.pack_dims))
         func = lambda x: x.tolist()
         ndata=ndata.map(func,res_type=node.type)
         return slice.modify(data=ndata,rtype=node.type,dims=node.dims)
@@ -614,6 +614,15 @@ class PyExec(VisitorFactory(prefixes=("visit",), flags=NF_ELSE),
 
     def numberGeneral(self, data, type_in, type_out, op):
         return numpy_unary_arith[op](data, sig=type_out.toNumpy())
+    
+    def boolInvert(self, data, type_in, type_out, op):
+        res = []
+        for elem in data:
+            if elem is Missing:
+                res.append(elem)
+            else:
+                res.append(not elem)
+        return cutils.darray(res, type_out.toNumpy())
 
     def corrCorr(self, data, type_in, type_out, op):
         intype = type_in.toNumpy()
